@@ -1,21 +1,32 @@
-import os, stat, re, errno
+import os
+import stat
+import re
+import errno
 
-from mercurial import dirstate
-from mercurial import hg
-from mercurial import ignore
-from mercurial import match as matchmod
-from mercurial import osutil
-from mercurial import scmutil
-# pathauditor moved to pathutil in 2.8
+from mercurial import (
+    dirstate,
+    match as matchmod,
+    osutil,
+    scmutil,
+    util,
+)
+
+try:
+    from mercurial import ignore
+    ignore.readpats
+    ignoremod = True
+except (AttributeError, ImportError):
+    # ignore module was removed in Mercurial 3.5
+    ignoremod = False
+# pathauditor moved to pathutil in 2.9
 try:
     from mercurial import pathutil
     pathutil.pathauditor
-except:
+except (AttributeError, ImportError):
     pathutil = scmutil
-from mercurial import util
 from mercurial.i18n import _
 
-def gignorepats(orig, lines, root = None):
+def gignorepats(orig, lines, root=None):
     '''parse lines (iterable) of .gitignore text, returning a tuple of
     (patterns, parse errors). These patterns should be given to compile()
     to be validated and converted into a match function.'''
@@ -46,27 +57,32 @@ def gignorepats(orig, lines, root = None):
             line = line[1:]
             rootsuffixes = ['']
         else:
-            rootsuffixes = ['','**/']
+            rootsuffixes = ['', '**/']
         for rootsuffix in rootsuffixes:
             pat = syntax + rootprefix + rootsuffix + line
             for s, rels in syntaxes.iteritems():
                 if line.startswith(rels):
                     pat = line
                     break
-                elif line.startswith(s+':'):
+                elif line.startswith(s + ':'):
                     pat = rels + line[len(s) + 1:]
                     break
             patterns.append(pat)
 
     return patterns, warnings
 
-def gignore(orig, root, files, warn, extrapatterns=None):
-    pats = ignore.readpats(root, files, warn)
+def gignore(root, files, warn, extrapatterns=None):
     allpats = []
+    pats = []
+    if ignoremod:
+        pats = ignore.readpats(root, files, warn)
+        for f, patlist in pats:
+            allpats.extend(patlist)
+    else:
+        allpats.extend(['include:%s' % f for f in files])
+
     if extrapatterns:
         allpats.extend(extrapatterns)
-    for f, patlist in pats:
-        allpats.extend(patlist)
     if not allpats:
         return util.never
     try:
@@ -90,9 +106,9 @@ class gitdirstate(dirstate.dirstate):
         files = [self._join('.hgignore')]
         for name, path in self._ui.configitems("ui"):
             if name == 'ignore' or name.startswith('ignore.'):
-                  files.append(util.expandpath(path))
+                files.append(util.expandpath(path))
         patterns = []
-        # Only use .gitignore if there's no .hgignore 
+        # Only use .gitignore if there's no .hgignore
         try:
             fp = open(files[0])
             fp.close()
@@ -101,29 +117,32 @@ class gitdirstate(dirstate.dirstate):
             for fn in fns:
                 d = os.path.dirname(fn)
                 fn = self.pathto(fn)
+                if not os.path.exists(fn):
+                    continue
                 fp = open(fn)
-                pats, warnings = gignorepats(None,fp,root=d)
+                pats, warnings = gignorepats(None, fp, root=d)
                 for warning in warnings:
                     self._ui.warn("%s: %s\n" % (fn, warning))
                 patterns.extend(pats)
-        return ignore.ignore(self._root, files, self._ui.warn, extrapatterns=patterns)
-    
+        return gignore(self._root, files, self._ui.warn,
+                             extrapatterns=patterns)
+
     def _finddotgitignores(self):
         """A copy of dirstate.walk. This is called from the new _ignore method,
-        which is called by dirstate.walk, which would cause infinite recursion, 
+        which is called by dirstate.walk, which would cause infinite recursion,
         except _finddotgitignores calls the superclass _ignore directly."""
-        match = matchmod.match(self._root, self.getcwd(), ['relglob:.gitignore'])
-        #TODO: need subrepos?
+        match = matchmod.match(self._root, self.getcwd(),
+                               ['relglob:.gitignore'])
+        # TODO: need subrepos?
         subrepos = []
         unknown = True
         ignored = False
-        full=True
 
         def fwarn(f, msg):
             self._ui.warn('%s: %s\n' % (self.pathto(f), msg))
             return False
 
-        ignore = super(gitdirstate,self)._ignore
+        ignore = super(gitdirstate, self)._ignore
         dirignore = self._dirignore
         if ignored:
             ignore = util.never
@@ -145,10 +164,10 @@ class gitdirstate(dirstate.dirstate):
         join = self._join
 
         exact = skipstep3 = False
-        if matchfn == match.exact: # match.exact
+        if matchfn == match.exact:  # match.exact
             exact = True
-            dirignore = util.always # skip step 2
-        elif match.files() and not match.anypats(): # match.match, no patterns
+            dirignore = util.always                  # skip step 2
+        elif match.files() and not match.anypats():  # match.match, no patterns
             skipstep3 = True
 
         if not exact and self._checkcase:
@@ -161,7 +180,11 @@ class gitdirstate(dirstate.dirstate):
         results, work, dirsnotfound = self._walkexplicit(match, subrepos)
 
         skipstep3 = skipstep3 and not (work or dirsnotfound)
-        work = [d for d in work if not dirignore(d)]
+        if work and isinstance(work[0], tuple):
+            # Mercurial >= 3.3.3
+            work = [nd for nd, d in work if not dirignore(d)]
+        else:
+            work = [d for d in work if not dirignore(d)]
         wadd = work.append
 
         # step 2: visit subdirectories
@@ -214,9 +237,10 @@ class gitdirstate(dirstate.dirstate):
             visit.sort()
 
             if unknown:
-                # unknown == True means we walked the full directory tree above.
-                # So if a file is not seen it was either a) not matching matchfn
-                # b) ignored, c) missing, or d) under a symlink directory.
+                # unknown == True means we walked the full directory tree
+                # above. So if a file is not seen it was either a) not matching
+                # matchfn b) ignored, c) missing, or d) under a symlink
+                # directory.
                 audit_path = pathutil.pathauditor(self._root)
 
                 for nf in iter(visit):
